define({
    'transitions': {
        'default': 'slide'
    },
    'pages': {
        'about': {
            'url': 'about',
            'label': 'About'
        },
        'item': {
            'url': 'items',
            'label': 'Items',
            'list': true,
            'map': true
        }
    },
    'map': {
        'zoom': 13,
        'center': [44.98, -93.27]
    }
});
